@@ -1,14 +1,16 @@
-# $Id: ReadLine.pm,v 1.13 2002/01/10 20:39:45 rcaputo Exp $
+# $Id: ReadLine.pm,v 1.15 2002/03/04 23:59:18 rcaputo Exp $
 
 package POE::Wheel::ReadLine;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = (qw($Revision: 1.13 $ ))[1];
+$VERSION = (qw($Revision: 1.15 $ ))[1];
 
 use Carp;
+use Symbol qw(gensym);
 use POE qw(Wheel);
+use POE::Preprocessor;
 
 # Things we'll need to interact with the terminal.
 use Term::Cap;
@@ -27,6 +29,13 @@ my ( $tck_up, $tck_down, $tck_left, $tck_right, $tck_insert,
 # Screen extent.
 my ($trk_rows, $trk_cols);
 
+# Private STDIN and STDOUT.
+my $stdin  = gensym();
+open($stdin, "<&0") or die "Can't open private STDIN";
+
+my $stdout = gensym;
+open($stdout, ">&1") or die "Can't open private STDOUT";
+
 # Offsets into $self.
 sub SELF_INPUT          () {  0 }
 sub SELF_CURSOR_INPUT   () {  1 }
@@ -75,6 +84,53 @@ BEGIN {
   }
 };
 
+# Wipe the current input line.  Implemented as a macro to avoid
+# circular closure problems with $self.
+
+macro wipe_input_line {
+  # Clear the current prompt and input, and home the cursor.
+  print $stdout $termcap->Tgoto( 'LE', 1,
+                         ( $$self_cursor_display + length($$self_prompt)
+                         )
+                       );
+  if ($tc_has_ke) {
+    print $stdout $termcap->Tputs( 'kE', 1 );
+  }
+  else {
+    my $wipe_length = length($$self_prompt) + display_width($$self_input);
+    print( $stdout
+           (' ' x $wipe_length), $termcap->Tgoto( 'LE', 1, $wipe_length)
+         );
+  }
+}
+
+# Helper to flush any buffered output.  A macro to avoid circular
+# closure problems with $self.
+
+macro flush_output_buffer {
+  # Flush anything buffered.
+  if (@$self_put_buffer) {
+    print $stdout @$self_put_buffer;
+
+    # Do not change the interior listref, or the event handlers will
+    # become confused.
+    @$self_put_buffer = ( );
+  }
+}
+
+# Set up the prompt and input line like nothing happened.  A macro to
+# avoid circular closure problems with $self.
+
+macro repaint_input_line {
+  print( $stdout $$self_prompt, normalize($$self_input) );
+  if ($$self_cursor_input != length($$self_input)) {
+    $termcap->Tgoto( 'LE', 1,
+                     ( display_width($$self_input) - $$self_cursor_display ),
+                     $stdout
+                   );
+  }
+}
+
 # Return a normalized version of a string.  This includes destroying
 # 8th-bit-set characters, turning them into strange multi-byte
 # sequences.  Apologies to everyone; please let me know of a portable
@@ -173,7 +229,7 @@ BEGIN {
   else    { $tck_backspace = preprocess_keystroke( 'kb' ); }
 
   # Terminal size.
-  ($trk_cols, $trk_rows) = Term::ReadKey::GetTerminalSize(*STDOUT);
+  ($trk_cols, $trk_rows) = Term::ReadKey::GetTerminalSize($stdout);
 
   # Esc is the generic meta prefix.
   $meta_prefix{chr(27)} = 1;
@@ -227,8 +283,8 @@ sub new {
         );
   }
 
-  # Turn off STDOUT buffering.
-  select((select(STDOUT), $| = 1)[0]);
+  # Turn off $stdout buffering.
+  select((select($stdout), $| = 1)[0]);
 
   # Set up console using Term::ReadKey.
   ReadMode('ultra-raw');
@@ -247,7 +303,7 @@ sub DESTROY {
   my $self = shift;
 
   # Stop selecting on the handle.
-  $poe_kernel->select( \*STDIN );
+  $poe_kernel->select($stdin);
 
   # Detach our tentacles from the parent session.
   if ($self->[SELF_STATE_READ]) {
@@ -276,9 +332,14 @@ sub _define_idle_state {
   my $self = shift;
 
   my $has_timer   = \$self->[SELF_HAS_TIMER];
-  my $put_buffer  = $self->[SELF_PUT_BUFFER];
   my $unique_id   = $self->[SELF_UNIQUE_ID];
 
+  my $self_put_buffer     = $self->[SELF_PUT_BUFFER];
+  my $self_cursor_display = \$self->[SELF_CURSOR_DISPLAY];
+  my $self_cursor_input   = \$self->[SELF_CURSOR_INPUT];
+  my $self_prompt         = \$self->[SELF_PROMPT];
+  my $self_input          = \$self->[SELF_INPUT];
+
   # This handler is called when input has become idle.
   $poe_kernel->state
     ( $self->[SELF_STATE_IDLE] = ref($self) . "($unique_id) -> input timeout",
@@ -288,10 +349,10 @@ sub _define_idle_state {
 
         my ($k, $s) = @_[KERNEL, SESSION];
 
-        if (@$put_buffer) {
-          $self->_wipe_input_line();
-          $self->_flush_output_buffer();
-          $self->_repaint_input_line();
+        if (@$self_put_buffer) {
+          {% wipe_input_line %}
+          {% flush_output_buffer %}
+          {% repaint_input_line %}
         }
 
         # No more timer.
@@ -334,6 +395,8 @@ sub _define_read_state {
     my $put_mode       = $self->[SELF_PUT_MODE];
     my $unique_id      = $self->[SELF_UNIQUE_ID];
 
+    my $self_put_buffer = $self->[SELF_PUT_BUFFER];
+
     $poe_kernel->state
       ( $self->[SELF_STATE_READ] = ref($self) . "($unique_id) -> select read",
         sub {
@@ -371,11 +434,11 @@ sub _define_read_state {
               # Beginning of line.
               if ( $key eq '^A' or $key eq $tck_home ) {
                 if ($$cursor_input) {
-                  $termcap->Tgoto( 'LE', 1, $$cursor_display, *STDOUT );
+                  $termcap->Tgoto( 'LE', 1, $$cursor_display, $stdout );
                   $$cursor_display = $$cursor_input = 0;
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -385,19 +448,19 @@ sub _define_read_state {
                 if ($$cursor_input) {
                   $$cursor_input--;
                   my $left = display_width(substr($$input, $$cursor_input, 1));
-                  $termcap->Tgoto( 'LE', 1, $left, *STDOUT );
+                  $termcap->Tgoto( 'LE', 1, $left, $stdout );
                   $$cursor_display -= $left;
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
 
               # Interrupt.
               if ($key eq '^C') {
-                print $key, "\x0D\x0A";
-                $poe_kernel->select_read( \*STDIN );
+                print $stdout $key, "\x0D\x0A";
+                $poe_kernel->select_read($stdin);
                 if ($$has_timer) {
                   $k->delay( $state_idle );
                   $$has_timer = 0;
@@ -407,7 +470,7 @@ sub _define_read_state {
                                   );
                 $$reading = 0;
                 $$hist_index = @$hist_list;
-                $self->_flush_output_buffer();
+                {% flush_output_buffer %}
                 next;
               }
 
@@ -421,11 +484,11 @@ sub _define_read_state {
                     ( normalize(substr($$input, $$cursor_input)) .
                       (' ' x $kill_width)
                     );
-                  print $normal;
-                  $termcap->Tgoto( 'LE', 1, length($normal), *STDOUT );
+                  print $stdout $normal;
+                  $termcap->Tgoto( 'LE', 1, length($normal), $stdout );
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -434,12 +497,12 @@ sub _define_read_state {
               if ( $key eq '^E' or $key eq $tck_end ) {
                 if ($$cursor_input < length($$input)) {
                   my $right = display_width(substr($$input, $$cursor_input));
-                  $termcap->Tgoto( 'RI', 1, $right, *STDOUT );
+                  $termcap->Tgoto( 'RI', 1, $right, $stdout );
                   $$cursor_display += $right;
                   $$cursor_input = length($$input);
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -448,20 +511,20 @@ sub _define_read_state {
               if ($key eq '^F' or $key eq $tck_right) {
                 if ($$cursor_input < length($$input)) {
                   my $normal = normalize(substr($$input, $$cursor_input, 1));
-                  print $normal;
+                  print $stdout $normal;
                   $$cursor_input++;
                   $$cursor_display += length($normal);
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
 
               # Cancel.
               if ($key eq '^G') {
-                print $key, "\x0D\x0A";
-                $poe_kernel->select_read( \*STDIN );
+                print $stdout $key, "\x0D\x0A";
+                $poe_kernel->select_read($stdin);
                 if ($$has_timer) {
                   $k->delay( $state_idle );
                   $$has_timer = 0;
@@ -471,7 +534,7 @@ sub _define_read_state {
                                   );
                 $$reading = 0;
                 $$hist_index = @$hist_list;
-                $self->_flush_output_buffer();
+                {% flush_output_buffer %}
                 return;
               }
 
@@ -483,25 +546,25 @@ sub _define_read_state {
                   my $kill_width =
                     display_width(substr($$input, $$cursor_input, 1));
                   substr($$input, $$cursor_input, 1) = '';
-                  $termcap->Tgoto( 'LE', 1, $left, *STDOUT );
+                  $termcap->Tgoto( 'LE', 1, $left, $stdout );
                   my $normal =
                     ( normalize(substr($$input, $$cursor_input)) .
                       (' ' x $kill_width)
                     );
-                  print $normal;
-                  $termcap->Tgoto( 'LE', 1, length($normal), *STDOUT );
+                  print $stdout $normal;
+                  $termcap->Tgoto( 'LE', 1, length($normal), $stdout );
                   $$cursor_display -= $kill_width;
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
 
               # Accept line.
               if ($key eq '^J') {
-                print "\x0D\x0A";
-                $poe_kernel->select_read( \*STDIN );
+                print $stdout "\x0D\x0A";
+                $poe_kernel->select_read($stdin);
                 if ($$has_timer) {
                   $k->delay( $state_idle );
                   $$has_timer = 0;
@@ -509,7 +572,7 @@ sub _define_read_state {
                 $poe_kernel->yield( $$event_input, $$input, $unique_id );
                 $$reading = 0;
                 $$hist_index = @$hist_list;
-                $self->_flush_output_buffer();
+                {% flush_output_buffer %}
                 next;
               }
 
@@ -519,12 +582,13 @@ sub _define_read_state {
                   my $kill_width =
                     display_width(substr($$input, $$cursor_input));
                   substr( $$input, $$cursor_input ) = '';
-                  print( (" " x $kill_width),
+                  print( $stdout
+                         (" " x $kill_width),
                          $termcap->Tgoto( 'LE', 1, $kill_width )
                        );
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -532,16 +596,16 @@ sub _define_read_state {
               # Clear screen.
               if ($key eq '^L') {
                 my $left = display_width(substr($$input, $$cursor_input));
-                $termcap->Tputs( 'cl', 1, *STDOUT );
-                print $$prompt, normalize($$input);
-                $termcap->Tgoto( 'LE', 1, $left, *STDOUT ) if $left;
+                $termcap->Tputs( 'cl', 1, $stdout );
+                print $stdout $$prompt, normalize($$input);
+                $termcap->Tgoto( 'LE', 1, $left, $stdout ) if $left;
                 next;
               }
 
               # Accept line.
               if ($key eq '^M') {
-                print "\x0D\x0A";
-                $poe_kernel->select_read( \*STDIN );
+                print $stdout "\x0D\x0A";
+                $poe_kernel->select_read($stdin);
                 if ($$has_timer) {
                   $k->delay( $state_idle );
                   $$has_timer = 0;
@@ -549,7 +613,7 @@ sub _define_read_state {
                 $poe_kernel->yield( $$event_input, $$input, $unique_id );
                 $$reading = 0;
                 $$hist_index = @$hist_list;
-                $self->_flush_output_buffer();
+                {% flush_output_buffer %}
                 next;
               }
 
@@ -563,12 +627,12 @@ sub _define_read_state {
                     reverse substr($$input, $$cursor_input - 1, 2);
                   substr($$input, $$cursor_input - 1, 2) = $transposition;
 
-                  $termcap->Tgoto( 'LE', 1, $width_left, *STDOUT );
-                  print normalize($transposition);
-                  $termcap->Tgoto( 'LE', 1, $width_left, *STDOUT );
+                  $termcap->Tgoto( 'LE', 1, $width_left, $stdout );
+                  print $stdout normalize($transposition);
+                  $termcap->Tgoto( 'LE', 1, $width_left, $stdout );
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -579,25 +643,25 @@ sub _define_read_state {
 
                   # Back up to the beginning of the line.
                   if ($$cursor_input) {
-                    print $termcap->Tgoto( 'LE', 1, $$cursor_display );
+                    print $stdout $termcap->Tgoto( 'LE', 1, $$cursor_display );
                     $$cursor_display = $$cursor_input = 0;
                   }
 
                   # Clear to the end of the line.
                   if ($tc_has_ke) {
-                    print $termcap->Tputs( 'kE', 1 );
+                    print $stdout $termcap->Tputs( 'kE', 1 );
                   }
                   else {
                     my $display_width = display_width($$input);
-                    print ' ' x $display_width;
-                    $termcap->Tgoto( 'LE', 1, $display_width, *STDOUT );
+                    print $stdout ' ' x $display_width;
+                    $termcap->Tgoto( 'LE', 1, $display_width, $stdout );
                   }
 
                   # Clear the input buffer.
                   $$input = '';
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -612,27 +676,27 @@ sub _define_read_state {
 
                   # Back up the screen cursor; show the line's tail.
                   my $delete_width = display_width($1);
-                  $termcap->Tgoto( 'LE', 1, $delete_width, *STDOUT );
-                  print normalize(substr( $$input, $$cursor_input ));
+                  $termcap->Tgoto( 'LE', 1, $delete_width, $stdout );
+                  print $stdout normalize(substr( $$input, $$cursor_input ));
 
                   # Clear to the end of the line.
                   if ($tc_has_ke) {
-                    print $termcap->Tputs( 'kE', 1 );
+                    print $stdout $termcap->Tputs( 'kE', 1 );
                   }
                   else {
-                    print ' ' x $delete_width;
-                    $termcap->Tgoto( 'LE', 1, $delete_width, *STDOUT );
+                    print $stdout ' ' x $delete_width;
+                    $termcap->Tgoto( 'LE', 1, $delete_width, $stdout );
                   }
 
                   # Back up the screen cursor to match the edit one.
                   if (length($$input) != $$cursor_input) {
                     my $display_width =
                       display_width( substr($$input, $$cursor_input) );
-                    $termcap->Tgoto( 'LE', 1, $display_width, *STDOUT );
+                    $termcap->Tgoto( 'LE', 1, $display_width, $stdout );
                   }
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -649,18 +713,18 @@ sub _define_read_state {
 
                   # Move cursor to start of input.
                   if ($$cursor_input) {
-                    $termcap->Tgoto( 'LE', 1, $$cursor_display, *STDOUT );
+                    $termcap->Tgoto( 'LE', 1, $$cursor_display, $stdout );
                   }
 
                   # Clear to end of line.
                   if (length $$input) {
                     if ($tc_has_ke) {
-                      print $termcap->Tputs( 'kE', 1 );
+                      print $stdout $termcap->Tputs( 'kE', 1 );
                     }
                     else {
                       my $display_width = display_width($$input);
-                      print ' ' x $display_width;
-                      $termcap->Tgoto( 'LE', 1, $display_width, *STDOUT );
+                      print $stdout ' ' x $display_width;
+                      $termcap->Tgoto( 'LE', 1, $display_width, $stdout );
                     }
                   }
 
@@ -668,14 +732,14 @@ sub _define_read_state {
                   # buffer, and show what the user's editing.  Set the
                   # cursor to the end of the new line.
                   my $normal;
-                  print $normal =
+                  print $stdout $normal =
                     normalize($$input = $hist_list->[--$$hist_index]);
                   $$cursor_input = length($$input);
                   $$cursor_display = length($normal);
                 }
                 else {
                   # At top of history list.
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -686,18 +750,18 @@ sub _define_read_state {
 
                   # Move cursor to start of input.
                   if ($$cursor_input) {
-                    $termcap->Tgoto( 'LE', 1, $$cursor_display, *STDOUT );
+                    $termcap->Tgoto( 'LE', 1, $$cursor_display, $stdout );
                   }
 
                   # Clear to end of line.
                   if (length $$input) {
                     if ($tc_has_ke) {
-                      print $termcap->Tputs( 'kE', 1 );
+                      print $stdout $termcap->Tputs( 'kE', 1 );
                     }
                     else {
                       my $display_width = display_width($$input);
-                      print ' ' x $display_width;
-                      $termcap->Tgoto( 'LE', 1, $display_width, *STDOUT );
+                      print $stdout ' ' x $display_width;
+                      $termcap->Tgoto( 'LE', 1, $display_width, $stdout );
                     }
                   }
 
@@ -705,12 +769,12 @@ sub _define_read_state {
                   if (++$$hist_index == @$hist_list) {
                     # Just past the end of the history.  Whatever was
                     # there when we left it.
-                    print $normal = normalize($$input = $$input_hold);
+                    print $stdout $normal = normalize($$input = $$input_hold);
                   }
                   else {
                     # There's something in the history list.  Make that
                     # the current line.
-                    print $normal =
+                    print $stdout $normal =
                       normalize($$input = $hist_list->[$$hist_index]);
                   }
 
@@ -718,7 +782,7 @@ sub _define_read_state {
                   $$cursor_display = length($normal);
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -735,32 +799,32 @@ sub _define_read_state {
 
                   # Move cursor to start of input.
                   if ($$cursor_input) {
-                    $termcap->Tgoto( 'LE', 1, $$cursor_display, *STDOUT );
+                    $termcap->Tgoto( 'LE', 1, $$cursor_display, $stdout );
                   }
 
                   # Clear to end of line.
                   if (length $$input) {
                     if ($tc_has_ke) {
-                      print $termcap->Tputs( 'kE', 1 );
+                      print $stdout $termcap->Tputs( 'kE', 1 );
                     }
                     else {
                       my $display_width = display_width($$input);
-                      print ' ' x $display_width;
-                      $termcap->Tgoto( 'LE', 1, $display_width, *STDOUT );
+                      print $stdout ' ' x $display_width;
+                      $termcap->Tgoto( 'LE', 1, $display_width, $stdout );
                     }
                   }
 
                   # Move the history cursor back, set the new input
                   # buffer, and show what the user's editing.  Set the
                   # cursor to the end of the new line.
-                  print my $normal =
+                  print $stdout my $normal =
                     normalize($$input = $hist_list->[$$hist_index = 0]);
                   $$cursor_input = length($$input);
                   $$cursor_display = length($normal);
                 }
                 else {
                   # At top of history list.
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -777,30 +841,30 @@ sub _define_read_state {
 
                   # Move cursor to start of input.
                   if ($$cursor_input) {
-                    $termcap->Tgoto( 'LE', 1, $$cursor_display, *STDOUT );
+                    $termcap->Tgoto( 'LE', 1, $$cursor_display, $stdout );
                   }
 
                   # Clear to end of line.
                   if (length $$input) {
                     if ($tc_has_ke) {
-                      print $termcap->Tputs( 'kE', 1 );
+                      print $stdout $termcap->Tputs( 'kE', 1 );
                     }
                     else {
                       my $display_width = display_width($$input);
-                      print ' ' x $display_width;
-                      $termcap->Tgoto( 'LE', 1, $display_width, *STDOUT );
+                      print $stdout ' ' x $display_width;
+                      $termcap->Tgoto( 'LE', 1, $display_width, $stdout );
                     }
                   }
 
                   # Move the edit line down to the last history line.
                   $$hist_index = @$hist_list - 1;
-                  print my $normal =
+                  print $stdout my $normal =
                     normalize($$input = $hist_list->[$$hist_index]);
                   $$cursor_input = length($$input);
                   $$cursor_display = length($normal);
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -821,12 +885,12 @@ sub _define_read_state {
                         ) = $word;
 
                   # Display the new text; move the cursor after it.
-                  print $space, normalize($word);
+                  print $stdout $space, normalize($word);
                   $$cursor_input += length($space . $word);
                   $$cursor_display += length($space) + display_width($word);
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -840,12 +904,12 @@ sub _define_read_state {
                   substr( $$input,
                           $$cursor_input + length($space), length($word)
                         ) = $word;
-                  print $space, normalize($word);
+                  print $stdout $space, normalize($word);
                   $$cursor_input += length($space . $word);
                   $$cursor_display += length($space) + display_width($word);
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -859,12 +923,12 @@ sub _define_read_state {
                   substr( $$input,
                           $$cursor_input + length($space), length($word)
                         ) = $word;
-                  print $space, normalize($word);
+                  print $stdout $space, normalize($word);
                   $$cursor_input += length($space . $word);
                   $$cursor_display += length($space) + display_width($word);
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -875,11 +939,11 @@ sub _define_read_state {
                 if (substr($$input, $$cursor_input) =~ /^(\s*\S+)/) {
                   $$cursor_input += length($1);
                   my $right = display_width($1);
-                  $termcap->Tgoto( 'RI', 1, $right, *STDOUT );
+                  $termcap->Tgoto( 'RI', 1, $right, $stdout );
                   $$cursor_display += $right;
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -892,22 +956,22 @@ sub _define_read_state {
 
                   my $normal_remaining =
                     normalize(substr($$input, $$cursor_input));
-                  print $normal_remaining;
+                  print $stdout $normal_remaining;
                   my $normal_remaining_length = length($normal_remaining);
 
                   if ($tc_has_ke) {
-                    print $termcap->Tputs( 'kE', 1 );
+                    print $stdout $termcap->Tputs( 'kE', 1 );
                   }
                   else {
-                    print ' ' x $killed_width;
+                    print $stdout ' ' x $killed_width;
                     $normal_remaining_length += $killed_width;
                   }
 
-                  $termcap->Tgoto( 'LE', 1, $normal_remaining_length, *STDOUT )
+                  $termcap->Tgoto( 'LE', 1, $normal_remaining_length, $stdout )
                     if $normal_remaining_length;
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -917,11 +981,11 @@ sub _define_read_state {
                 if (substr($$input, 0, $$cursor_input) =~ /(\S+\s*)$/) {
                   $$cursor_input -= length($1);
                   my $kill_width = display_width($1);
-                  $termcap->Tgoto( 'LE', 1, $kill_width, *STDOUT );
+                  $termcap->Tgoto( 'LE', 1, $kill_width, $stdout );
                   $$cursor_display -= $kill_width;
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                 }
                 next;
               }
@@ -966,7 +1030,7 @@ sub _define_read_state {
                   }
                 }
                 else {
-                  print $tc_bell;
+                  print $stdout $tc_bell;
                   next;
                 }
 
@@ -978,10 +1042,10 @@ sub _define_read_state {
                 if ($$cursor_display - display_width($previous)) {
                   $termcap->Tgoto( 'LE', 1,
                                    $$cursor_display - display_width($previous),
-                                   *STDOUT
+                                   $stdout
                                  );
                 }
-                print normalize($right . $space . $left);
+                print $stdout normalize($right . $space . $left);
                 $$cursor_input = length($previous. $left . $space . $right);
                 $$cursor_display =
                   display_width($previous . $left . $space . $right);
@@ -999,13 +1063,14 @@ sub _define_read_state {
             # C-q displays some stuff.
             if ($key eq '^Q') {
               my $left = display_width(substr($$input, $$cursor_input));
-              print( "\x0D\x0A",
+              print( $stdout
+                     "\x0D\x0A",
                      "cursor_input($$cursor_input) ",
                      "cursor_display($$cursor_display) ",
                      "term_columns($trk_cols)\x0D\x0A",
                      $$prompt, normalize($$input)
                    );
-              $termcap->Tgoto( 'LE', 1, $left, *STDOUT ) if $left;
+              $termcap->Tgoto( 'LE', 1, $left, $stdout ) if $left;
               next;
             }
 
@@ -1013,7 +1078,7 @@ sub _define_read_state {
             # function key or something.  Don't allow it to be
             # entered.
             if (length($raw_key) > 1) {
-              print $tc_bell;
+              print $stdout $tc_bell;
               next;
             }
 
@@ -1026,10 +1091,10 @@ sub _define_read_state {
                 # Insert.
                 my $normal = normalize(substr($$input, $$cursor_input));
                 substr($$input, $$cursor_input, 0) = $raw_key;
-                print $key, $normal;
+                print $stdout $key, $normal;
                 $$cursor_input += length($raw_key);
                 $$cursor_display += length($key);
-                $termcap->Tgoto( 'LE', 1, length($normal), *STDOUT );
+                $termcap->Tgoto( 'LE', 1, length($normal), $stdout );
               }
               else {
                 # Overstrike.
@@ -1039,7 +1104,7 @@ sub _define_read_state {
                     );
                 substr($$input, $$cursor_input, length($raw_key)) = $raw_key;
 
-                print $key;
+                print $stdout $key;
                 $$cursor_input += length($raw_key);
                 $$cursor_display += length($key);
 
@@ -1050,14 +1115,14 @@ sub _define_read_state {
                   if (length($key) < $replaced_width) {
                     $rest .= ' ' x ($replaced_width - length($key));
                   }
-                  print $rest;
-                  $termcap->Tgoto( 'LE', 1, length($rest), *STDOUT );
+                  print $stdout $rest;
+                  $termcap->Tgoto( 'LE', 1, length($rest), $stdout );
                 }
               }
             }
             else {
               # Append.
-              print $key;
+              print $stdout $key;
               $$input .= $raw_key;
               $$cursor_input += length($raw_key);
               $$cursor_display += length($key);
@@ -1067,12 +1132,12 @@ sub _define_read_state {
       );
 
     # Now select on it.
-    $poe_kernel->select_read( \*STDIN, $self->[SELF_STATE_READ] );
+    $poe_kernel->select_read($stdin, $self->[SELF_STATE_READ]);
   }
 
   # Otherwise we're undefining it.
   else {
-    $poe_kernel->select_read( \*STDIN );
+    $poe_kernel->select_read($stdin);
   }
 
 }
@@ -1094,60 +1159,9 @@ sub get {
   $self->[SELF_INSERT_MODE]    = 1;
 
   # Watch the filehandle.
-  $poe_kernel->select( \*STDIN, $self->[SELF_STATE_READ] );
+  $poe_kernel->select($stdin, $self->[SELF_STATE_READ]);
 
-  print $prompt;
-}
-
-# Helper to wipe the current input line.
-sub _wipe_input_line {
-  my $self = shift;
-
-  # Clear the current prompt and input, and home the cursor.
-  print $termcap->Tgoto( 'LE', 1,
-                         ( $self->[SELF_CURSOR_DISPLAY] +
-                           length($self->[SELF_PROMPT])
-                         )
-                       );
-  if ($tc_has_ke) {
-    print $termcap->Tputs( 'kE', 1 );
-  }
-  else {
-    my $wipe_length =
-      ( length($self->[SELF_PROMPT]) +
-        display_width($self->[SELF_INPUT])
-      );
-    print( (' ' x $wipe_length), $termcap->Tgoto( 'LE', 1, $wipe_length) );
-  }
-}
-
-# Helper to flush any buffered output.
-sub _flush_output_buffer {
-  my $self = shift;
-
-  # Flush anything buffered.
-  if (@{$self->[SELF_PUT_BUFFER]}) {
-    print @{$self->[SELF_PUT_BUFFER]};
-
-    # Do not change the interior listref, or the event handlers will
-    # become confused.
-    @{$self->[SELF_PUT_BUFFER]} = ( );
-  }
-}
-
-# Set up the prompt and input line like nothing happened.
-sub _repaint_input_line {
-  my $self = shift;
-
-  print( $self->[SELF_PROMPT], normalize($self->[SELF_INPUT]) );
-  if ($self->[SELF_CURSOR_INPUT] != length($self->[SELF_INPUT])) {
-    $termcap->Tgoto( 'LE', 1,
-                     ( display_width($self->[SELF_INPUT]) -
-                       $self->[SELF_CURSOR_DISPLAY]
-                     ),
-                     *STDOUT
-                   );
-  }
+  print $stdout $prompt;
 }
 
 # Write a line on the terminal.
@@ -1170,14 +1184,22 @@ sub put {
     #         "timer($self->[SELF_HAS_TIMER]) "
     #       );
 
-    $self->_wipe_input_line();
-    $self->_flush_output_buffer();
+    my $self_put_buffer     = $self->[SELF_PUT_BUFFER];
+    my $self_cursor_display = \$self->[SELF_CURSOR_DISPLAY];
+    my $self_prompt         = \$self->[SELF_PROMPT];
+    my $self_input          = \$self->[SELF_INPUT];
+    my $self_cursor_input   = \$self->[SELF_CURSOR_INPUT];
+
+    {% wipe_input_line %}
+    {% flush_output_buffer %}
 
     # Print the new stuff.
-    print @lines;
+    print $stdout @lines;
 
     # Only repaint the input if we're reading a line.
-    $self->_repaint_input_line() if $self->[SELF_READING_LINE];
+    if ($self->[SELF_READING_LINE]) {
+      {% repaint_input_line %}
+    }
 
     return;
   }
@@ -1195,7 +1217,7 @@ sub put {
 # Clear the screen.
 sub clear {
   my $self = shift;
-  $termcap->Tputs( cl => 1, *STDOUT );
+  $termcap->Tputs( cl => 1, $stdout );
 }
 
 # Add things to the edit history.
