@@ -1,11 +1,11 @@
-# $Id: SocketFactory.pm,v 1.53 2002/01/10 20:39:45 rcaputo Exp $
+# $Id: SocketFactory.pm,v 1.54 2002/02/25 06:02:58 rcaputo Exp $
 
 package POE::Wheel::SocketFactory;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = (qw($Revision: 1.53 $ ))[1];
+$VERSION = (qw($Revision: 1.54 $ ))[1];
 
 use Carp;
 use Symbol;
@@ -506,9 +506,9 @@ sub new {
   # Configure Socket #
   #------------------#
 
-  # Make the socket binary.  This probably is necessary for DOSISH
-  # systems, and nothing untoward should happen on sane systems.
-  binmode($socket_handle);
+  # Make the socket binary.  It's wrapped in eval{} because tied
+  # filehandle classes may actually die in their binmode methods.
+  eval { binmode($socket_handle) };
 
   # Don't block on socket operations, because the socket will be
   # driven by a select loop.
